@@ -18,10 +18,10 @@ from .._utils import in_context, patch_showwarnings
 from . import __name__ as __parent__
 from ._base import Command, CommandExit, InvalidArgument
 
-try:
-    import importlib.metadata as importlib_metadata
-except ImportError:
+if sys.version_info < (3, 10):
     import importlib_metadata  # type: ignore
+else:
+    import importlib.metadata as importlib_metadata
 
 
 class Main(Command):
@@ -33,7 +33,11 @@ class Main(Command):
     @classmethod
     def _entry_points(cls) -> List["importlib_metadata.EntryPoint"]:
         if cls._entry_points_cache is None:
-            cls._entry_points_cache = list(importlib_metadata.entry_points().get(__parent__, []))
+            try:
+                entry_points = importlib_metadata.entry_points(group=__parent__)
+                cls._entry_points_cache = list(entry_points)
+            except KeyError:
+                cls._entry_points_cache = []
         return cls._entry_points_cache
 
     @classmethod
@@ -76,7 +80,7 @@ class Main(Command):
         gecco - Gene Cluster Prediction with Conditional Random Fields
 
         Usage:
-            gecco [-v | -vv | -q | -qq] <cmd> [<args>...]
+            gecco [-v | -vv | -q | -qq] [--no-progress-bar] <cmd> [<args>...]
             gecco --version
             gecco --help [<cmd>]
 
@@ -91,6 +95,8 @@ class Main(Command):
             -v, --verbose              increase verbosity (-v is verbose,
                                        -vv is very verbose and makes the
                                        output more suitable for logging).
+            --no-progress-bar          disable progress bars from the output,
+                                       independently of verbosity.
             -V, --version              show the program version and exit.
 
         """
@@ -148,7 +154,7 @@ class Main(Command):
                 )
                 subcmd.verbose = self.verbose
                 subcmd.quiet = self.quiet
-                subcmd.progress.disable = self.quiet > 0
+                subcmd.progress.disable = self.args["--no-progress-bar"]
             # run the subcommand
             return subcmd.execute(ctx)
         except CommandExit as sysexit:
