@@ -156,7 +156,7 @@ class PyHMMER(DomainAnnotator):
                     start = domain.alignment.target_from
                     end = domain.alignment.target_to
 
-                    # extract qualifiers
+                    # extract qualifiers and GO terms
                     qualifiers: Dict[str, List[str]] = {
                         "inference": ["protein motif"],
                         "db_xref": ["{}:{}".format(self.hmm.id.upper(), accession)],
@@ -167,15 +167,30 @@ class PyHMMER(DomainAnnotator):
                     }
                     if entry is not None:
                         qualifiers["function"] = [entry.name]
-                    if entry is not None and entry.integrated is not None:
-                        qualifiers["db_xref"].append("InterPro:{}".format(entry.integrated))
+                        qualifiers["db_xref"].append("InterPro:{}".format(entry.accession))
+                        go_terms = entry.go_terms
+                        go_functions = entry.go_functions
+                    else:
+                        go_terms = []
+                        go_functions = []
 
                     # add the domain to the protein domains of the right gene
                     assert domain.env_from < domain.env_to
                     assert domain.i_evalue >= 0
                     assert domain.pvalue >= 0
-                    d = Domain(accession, start, end, self.hmm.id, domain.i_evalue, domain.pvalue, None, qualifiers)
-                    gene_index[target_index].protein.domains.append(d)
+                    gene_index[target_index].protein.domains.append(
+                        Domain(
+                            accession,
+                            start,
+                            end,
+                            self.hmm.id,
+                            domain.i_evalue,
+                            domain.pvalue,
+                            go_terms=go_terms,
+                            go_functions=go_functions,
+                            qualifiers=qualifiers,
+                        )
+                    )
 
         # deduplicate hits per gene for exclusive HMMs
         if self.hmm.exclusive:
