@@ -1,7 +1,8 @@
-"""Data layer classes storing information needed for BGC detection.
+"""Data layer classes storing information needed for gene cluster detection.
 """
 
 import collections
+import copy
 import csv
 import datetime
 import enum
@@ -10,12 +11,13 @@ import itertools
 import math
 import operator
 import re
+import statistics
 import typing
 from array import array
 from collections import OrderedDict
 from collections.abc import Sized
 from dataclasses import dataclass, field
-from typing import Dict, Iterable, List, Mapping, Optional, Sequence, TextIO, NamedTuple, Union, Iterator
+from typing import Dict, Iterable, List, Mapping, Optional, Sequence, TextIO, NamedTuple, Union, Iterator, Set
 
 import Bio
 import numpy
@@ -24,6 +26,7 @@ from Bio.SeqFeature import SeqFeature, FeatureLocation, CompoundLocation, Refere
 from Bio.SeqRecord import SeqRecord
 
 from . import __version__
+from .interpro import GOTerm
 from ._base import Dumpable, Table, _SELF
 from ._meta import patch_locale
 
@@ -32,7 +35,7 @@ if typing.TYPE_CHECKING:
 
 
 __all__ = [
-    "ProductType",
+    "ClusterType",
     "Strand",
     "Domain",
     "Protein",
@@ -44,45 +47,45 @@ __all__ = [
 
 
 # fmt: off
-class ProductType(object):
-    """An immutable storage of type for the product synthesized by a cluster.
+class ClusterType(object):
+    """An immutable storage for the type of a gene cluster.
     """
 
     def __init__(self, *names: str) -> None:
         """Create a new product type from one or more base types.
 
         Example:
-            >>> t1 = ProductType()                    # unknown type
-            >>> t2 = ProductType("Polyketide")        # single type
-            >>> t3 = ProductType("Polyketide", "NRP") # multiple types
+            >>> t1 = ClusterType()                    # unknown type
+            >>> t2 = ClusterType("Polyketide")        # single type
+            >>> t3 = ClusterType("Polyketide", "NRP") # multiple types
 
         """
         self.names = frozenset(names)
 
     def __repr__(self) -> str:  # noqa: D105
-        return "ProductType({})".format(", ".join(map(repr, sorted(self.names))))
+        return "ClusterType({})".format(", ".join(map(repr, sorted(self.names))))
 
     def __hash__(self) -> int:  # noqa: D105
         return hash(self.names)
 
     def __eq__(self, other: object) -> bool:  # noqa: D105
-        if not isinstance(other, ProductType):
+        if not isinstance(other, ClusterType):
             return NotImplemented
         return self.names == other.names
 
     def __bool__(self) -> bool:  # noqa: D105
         return len(self.names) != 0
 
-    def unpack(self) -> List["ProductType"]:
-        """Unpack a composite `ProductType` into a list of individual types.
+    def unpack(self) -> List["ClusterType"]:
+        """Unpack a composite `ClusterType` into a list of individual types.
 
         Example:
-            >>> ty = ProductType("Polyketide", "Saccharide")
+            >>> ty = ClusterType("Polyketide", "Saccharide")
             >>> ty.unpack()
-            [ProductType('Polyketide'), ProductType('Saccharide')]
+            [ClusterType('Polyketide'), ClusterType('Saccharide')]
 
         """
-        return [ ProductType(x) for x in sorted(self.names) ]
+        return [ ClusterType(x) for x in sorted(self.names) ]
 
 
 class Strand(enum.IntEnum):
@@ -116,8 +119,19 @@ class Domain:
         pvalue (`float`): The p-value reported by ``hmmsearch`` that measure
             how likely the domain score is.
         probability (`float`, optional): The probability that this domain
-            is part of a BGC, or `None` if no prediction has been made yet.
-        qualifiers (`dict`, optional): A dictionary of feature qualifiers that
+            is part of a gene cluster, or `None` if no prediction has been 
+            made yet.
+        cluster_weight (`float`, optional): The weight for this domain, 
+            measuring its importance as infered from the training clusters
+            by the CRF model.
+        go_terms (`list` of `GOTerm`): The Gene Ontology terms
+            for this particular domain.
+        go_functions (`list` of `GOTerm`): The Gene Ontology term families for 
+            this particular domain. Term families are extracted by taking the 
+            highest superclasses (excluding the root) of each Gene Ontology 
+            term in the ``molecular_function`` namespace associated with this 
+            domain.
+        qualifiers (`dict`): A dictionary of feature qualifiers that
             is added to the `~Bio.SeqFeature.SeqFeature` built from this
             `Domain`.
 
@@ -130,14 +144,33 @@ class Domain:
     i_evalue: float
     pvalue: float
     probability: Optional[float] = None
+    cluster_weight: Optional[float] = None
+    go_terms: List[GOTerm] = field(default_factory=list)
+    go_functions: List[GOTerm] = field(default_factory=list)
     qualifiers: Dict[str, List[str]] = field(default_factory=dict)
 
     def with_probability(self, probability: Optional[float]) -> "Domain":
-        """Copy the current domain and assign it a BGC probability.
+        """Copy the current domain and assign it a cluster probability.
         """
         return Domain(
             self.name, self.start, self.end, self.hmm, self.i_evalue, self.pvalue,
-            probability, self.qualifiers.copy()
+            probability,
+            self.cluster_weight,
+            self.go_terms,
+            self.go_functions,
+            self.qualifiers.copy()
+        )
+
+    def with_cluster_weight(self, cluster_weight: Optional[float]) -> "Domain":
+        """Copy the current domain and assign it a cluster weight.
+        """
+        return Domain(
+            self.name, self.start, self.end, self.hmm, self.i_evalue, self.pvalue,
+            self.probability,
+            cluster_weight,
+            self.go_terms,
+            self.go_functions,
+            self.qualifiers.copy()
         )
 
     def to_seq_feature(self, protein_coordinates: bool = False) -> SeqFeature:
@@ -153,6 +186,8 @@ class Domain:
         loc = FeatureLocation((self.start-1)*stride, self.end*stride)
         qualifiers = dict(self.qualifiers)
         qualifiers.setdefault("standard_name", [self.name])
+        for go_term in self.go_terms:
+            qualifiers.setdefault("db_xref", []).append(go_term.accession)
         return SeqFeature(location=loc, type="misc_feature", qualifiers=qualifiers)
 
 
@@ -186,6 +221,11 @@ class Protein:
 
         return record
 
+    def with_seq(self, seq: Seq) -> "Protein":
+        """Copy the current protein and assign it a new sequence.
+        """
+        return Protein(self.id, seq, copy.deepcopy(self.domains))
+
     def with_domains(self, domains: Iterable[Domain]) -> "Protein":
         """Copy the current protein and assign it new domains.
         """
@@ -228,16 +268,16 @@ class Gene:
 
     @property
     def average_probability(self) -> Optional[float]:
-        """`float`: The average of domain probabilities of being biosynthetic.
+        """`float`: The average of domain probabilities of being in a cluster.
         """
         if self._probability is not None:
             return self._probability
         p = [d.probability for d in self.protein.domains if d.probability is not None]
-        return sum(p) / len(p) if p else None
+        return statistics.mean(p) if p else None
 
     @property
     def maximum_probability(self) -> Optional[float]:
-        """`float`: The highest of domain probabilities of being biosynthetic.
+        """`float`: The highest of domain probabilities of being in a cluster.
         """
         if self._probability is not None:
             return self._probability
@@ -246,7 +286,31 @@ class Gene:
 
     # ---
 
-    def to_seq_feature(self) -> SeqFeature:
+    # NB(@althonos): Color scheme taken from MIBiG.
+    #                This is sorted by priority!
+    _FUNCTION_PALETTE = {
+        # transporter: blue
+        "transporter activity": (0x64, 0x95, 0xed),
+        "cargo receptor activity": (0x64, 0x95, 0xed),
+        "molecular carrier activity": (0x64, 0x95, 0xed),
+        # regulatory: green
+        "translation regulator activity": (0x2e, 0x8b, 0x56),
+        "molecular function regulator activity": (0x2e, 0x8b, 0x56),
+        "transcription regulator activity": (0x2e, 0x8b, 0x56),
+        "regulation of molecular function": (0x2e, 0x8b, 0x56),
+        "general transcription initiation factor activity": (0x2e, 0x8b, 0x56),
+        # core biosynthetic: red
+        "toxin activity": (0x81, 0x0e, 0x15),
+        "catalytic activity": (0x81, 0x0e, 0x15),
+        # additional biosynthetic: pink
+        "biosynthetic activity": (0xf1, 0x6d, 0x75),
+        # non-biosynthetic: olive green
+        "non-biosynthetic activity": (0xbd, 0xb7, 0x6b),
+        # unknown: grey
+        "unknown": (0x80, 0x80, 0x80),
+    }
+
+    def to_seq_feature(self, color: bool = True) -> SeqFeature:
         """Convert the gene to a single feature.
         """
         # NB(@althonos): we use inclusive 1-based ranges in the data model
@@ -255,6 +319,23 @@ class Gene:
         qualifiers = dict(self.qualifiers)
         qualifiers.setdefault("locus_tag", [self.protein.id])
         qualifiers.setdefault("translation", [str(self.protein.seq)])
+
+        # NB(@althonos): Attempt to assign a function for the gene based on the
+        #                domain content.
+        functions = self.functions()
+        qualifiers.setdefault("function", sorted(functions))
+        if color:
+            for k, hexcolor in self._FUNCTION_PALETTE.items():
+                if k in functions:
+                    break
+            else:
+                hexcolor = self._FUNCTION_PALETTE["unknown"]
+            # EasyFig qualifier
+            qualifiers.setdefault("colour", [" ".join(str(x) for x in hexcolor)])
+            # SnapGene qualifiers
+            qualifiers.setdefault("ApEinfo_fwdcolor", ["#{:02x}{:02x}{:02x}".format(*hexcolor)])
+            qualifiers.setdefault("ApEinfo_revcolor", ["#{:02x}{:02x}{:02x}".format(*hexcolor)])
+
         return SeqFeature(location=loc, type="CDS", qualifiers=qualifiers)
 
     def with_protein(self, protein: "Protein") -> "Gene":
@@ -288,34 +369,46 @@ class Gene:
             _probability=probability
         )
 
+    def functions(self) -> Set[str]:
+        """Predict the function(s) of the gene from its domain annotations.
+        """
+        functions = {
+            term.name
+            for domain in self.protein.domains
+            for term in domain.go_functions
+        }
+        if not functions:
+            functions.add("unknown")
+        return functions
+
 
 @dataclass
 class Cluster:
-    """A sequence of contiguous genes with biosynthetic activity.
+    """A sequence of contiguous genes.
 
     Attributes:
         id (`str`): The identifier of the gene cluster.
         genes (`list` of `~gecco.model.Gene`): A list of the genes belonging
             to this gene cluster.
-        types (`gecco.model.ProductType`): The putative types of product
-            synthesized by this gene cluster, according to similarity in
-            domain composition with curated clusters.
+        types (`gecco.model.ClusterType`): The putative types of this gene 
+            cluster, according to similarity in domain composition with 
+            curated clusters.
         types_probabilities (`list` of `float`): The probability with which
-            each BGC type was identified (same dimension as the ``types``
+            each cluster type was identified (same dimension as the ``types``
             attribute).
 
     """
 
     id: str
     genes: List[Gene]
-    type: ProductType
+    type: ClusterType
     type_probabilities: Dict[str, float]
 
     def __init__(
         self,
         id: str,
         genes: Optional[List[Gene]] = None,
-        type: ProductType = ProductType(),
+        type: ClusterType = ClusterType(),
         type_probabilities: Optional[Dict[str, float]] = None,
     ):  # noqa: D107
         self.id = id
@@ -346,7 +439,7 @@ class Cluster:
         """`float`: The average of proteins probability of being biosynthetic.
         """
         p = [g.average_probability for g in self.genes if g.average_probability is not None]
-        return sum(p) / len(p) if p else None
+        return  statistics.mean(p) if p else None
 
     @property
     def maximum_probability(self) -> Optional[float]:
@@ -369,7 +462,7 @@ class Cluster:
         Arguments:
             all_possible (sequence of `str`, optional): A sequence containing
                 all domain names to consider when computing domain composition
-                for the BGC. If `None` given, then only domains within the
+                for the cluster. If `None` given, then only domains within the
                 cluster are taken into account.
             normalize (`bool`): Normalize the composition vector so that it
                 sums to 1.
@@ -382,7 +475,7 @@ class Cluster:
 
         Returns:
             `~numpy.ndarray`: A numerical array containing the relative domain
-            composition of the BGC.
+            composition of the gene cluster.
 
         """
         domains = [d for gene in self.genes for d in gene.protein.domains]
@@ -420,21 +513,21 @@ class Cluster:
 
         # NB(@althonos): we use inclusive 1-based ranges in the data model
         # but slicing expects 0-based ranges with exclusive ends
-        bgc = self.source[self.start - 1 : self.end]
-        bgc.id = bgc.name = self.id
+        cluster = self.source[self.start - 1 : self.end]
+        cluster.id = cluster.name = self.id
 
         # copy sequence annotations
-        bgc.annotations = self.source.annotations.copy()
-        bgc.annotations["topology"] = "linear"
-        bgc.annotations["molecule_type"] = "DNA"
+        cluster.annotations = self.source.annotations.copy()
+        cluster.annotations["topology"] = "linear"
+        cluster.annotations["molecule_type"] = "DNA"
         with patch_locale("C"):
-            bgc.annotations['date'] = now.strftime("%d-%b-%Y").upper()
+            cluster.annotations['date'] = now.strftime("%d-%b-%Y").upper()
 
         biopython_version = tuple(map(int, Bio.__version__.split(".")))
         if biopython_version < (1, 77):
             from Bio import Alphabet
 
-            bgc.seq.alphabet = Alphabet.generic_dna
+            cluster.seq.alphabet = Alphabet.generic_dna
 
         # add GECCO preprint as a reference
         ref = Reference()
@@ -450,18 +543,18 @@ class Cluster:
             "Elisa Cappio Barazzone",
             "Georg Zeller"
         ])
-        bgc.annotations.setdefault("references", []).append(ref)
+        cluster.annotations.setdefault("references", []).append(ref)
 
         # add GECCO-specific annotations as a structured comment
         probabilities = {
             f"{key.lower()}_probability":f"{value:.3f}"
             for key, value in self.type_probabilities.items()
         }
-        structured_comment = bgc.annotations.setdefault("structured_comment", OrderedDict())
+        structured_comment = cluster.annotations.setdefault("structured_comment", OrderedDict())
         structured_comment['GECCO-Data'] = {
             "version": f"GECCO v{__version__}",
             "creation_date": now.isoformat(),
-            "biosyn_class": ";".join(sorted(self.type.names)) or "Unknown",
+            "cluster_type": ";".join(sorted(self.type.names)) or "Unknown",
             **probabilities,
         }
 
@@ -470,15 +563,24 @@ class Cluster:
             # write gene as a /cds GenBank record
             cds = gene.to_seq_feature()
             cds.location += -self.start
-            bgc.features.append(cds)
+            cluster.features.append(cds)
             # write domains as /misc_feature annotations
             for domain in gene.protein.domains:
                 misc = domain.to_seq_feature(protein_coordinates=False)
-                misc.location += cds.location.start
-                bgc.features.append(misc)
+                if gene.strand == Strand.Coding:
+                    misc.location = FeatureLocation(
+                        cds.location.start + misc.location.start,
+                        cds.location.start + misc.location.end
+                    )
+                else:
+                    misc.location = FeatureLocation(
+                        cds.location.end - misc.location.end,
+                        cds.location.end - misc.location.start
+                    )
+                cluster.features.append(misc)
 
-        # return the complete BGC
-        return bgc
+        # return the complete gene cluster record
+        return cluster
 
 
 class _UnknownSeq(Seq):
@@ -520,7 +622,7 @@ class FeatureTable(Table):
     pvalue: List[float] = field(default_factory = lambda: array("d"))       # type: ignore
     domain_start: List[int] = field(default_factory = lambda: array("l"))   # type: ignore
     domain_end: List[int] = field(default_factory = lambda: array("l"))     # type: ignore
-    bgc_probability: List[Optional[float]] = field(default_factory = list)
+    cluster_probability: List[Optional[float]] = field(default_factory = list)
 
     class Row(NamedTuple):
         """A single row in a feature table.
@@ -537,7 +639,7 @@ class FeatureTable(Table):
         pvalue: float
         domain_start: int
         domain_end: int
-        bgc_probability: Optional[float]
+        cluster_probability: Optional[float]
 
     if typing.TYPE_CHECKING:
 
@@ -573,7 +675,7 @@ class FeatureTable(Table):
                 table.pvalue.append(domain.pvalue)
                 table.domain_start.append(domain.start)
                 table.domain_end.append(domain.end)
-                table.bgc_probability.append(domain.probability)
+                table.cluster_probability.append(domain.probability)
         return table
 
     def to_genes(self) -> Iterable[Gene]:
@@ -599,7 +701,7 @@ class FeatureTable(Table):
             protein = Protein(rows[0].protein_id, seq=_UnknownSeq)
             gene = Gene(source, rows[0].start, rows[0].end, strand, protein)
             for row in rows:
-                domain = Domain(row.domain, row.domain_start, row.domain_end, row.hmm, row.i_evalue, row.pvalue, row.bgc_probability)
+                domain = Domain(row.domain, row.domain_start, row.domain_end, row.hmm, row.i_evalue, row.pvalue, row.cluster_probability)
                 gene.protein.domains.append(domain)
             yield gene
 
@@ -607,12 +709,12 @@ class FeatureTable(Table):
         return len(self.sequence_id)
 
 
-def _format_product_type(value: "ProductType") -> str:
+def _format_product_type(value: "ClusterType") -> str:
     return ";".join(sorted(value.names))
 
 
-def _parse_product_type(value: str) -> "ProductType":
-    return ProductType(*map(str.strip, value.split(";"))) if value.strip() else ProductType()
+def _parse_product_type(value: str) -> "ClusterType":
+    return ClusterType(*map(str.strip, value.split(";"))) if value.strip() else ClusterType()
 
 
 @dataclass(frozen=True)
@@ -621,13 +723,13 @@ class ClusterTable(Table):
     """
 
     sequence_id: List[str] = field(default_factory = list)
-    bgc_id: List[str] = field(default_factory = list)
+    cluster_id: List[str] = field(default_factory = list)
     start: List[int] = field(default_factory = lambda: array("l"))   # type: ignore
     end: List[int] = field(default_factory = lambda: array("l"))     # type: ignore
     average_p: List[Optional[float]] = field(default_factory = list)
     max_p: List[Optional[float]] = field(default_factory = list)
 
-    type: List[ProductType] = field(default_factory = list)
+    type: List[ClusterType] = field(default_factory = list)
     type_p: List[Dict[str, float]] = field(default_factory = list)
 
     proteins: List[Optional[List[str]]] = field(default_factory = list)
@@ -638,20 +740,20 @@ class ClusterTable(Table):
         """
 
         sequence_id: str
-        bgc_id: str
+        cluster_id: str
         start: int
         end: int
         average_p: Optional[float]
         max_p: Optional[float]
 
-        type: ProductType
-        type_p: Dict[ProductType, float]
+        type: ClusterType
+        type_p: Dict[ClusterType, float]
 
         proteins: Optional[List[str]]
         domains: Optional[List[str]]
 
-    _FORMAT_FIELD = {ProductType: _format_product_type, **Table._FORMAT_FIELD}
-    _PARSE_FIELD = {ProductType: _parse_product_type, **Table._PARSE_FIELD}
+    _FORMAT_FIELD = {ClusterType: _format_product_type, **Table._FORMAT_FIELD}
+    _PARSE_FIELD = {ClusterType: _parse_product_type, **Table._PARSE_FIELD}
 
     @classmethod
     def from_clusters(cls, clusters: Iterable[Cluster]) -> "ClusterTable":
@@ -660,7 +762,7 @@ class ClusterTable(Table):
         table = cls()
         for cluster in clusters:
             table.sequence_id.append(cluster.source.id)
-            table.bgc_id.append(cluster.id)
+            table.cluster_id.append(cluster.id)
             table.start.append(cluster.start)
             table.end.append(cluster.end)
             table.average_p.append(cluster.average_probability)
@@ -768,7 +870,7 @@ class ClusterTable(Table):
 
 @dataclass(frozen=True)
 class GeneTable(Table):
-    """A table storing gene coordinates and optional biosynthetic probabilities.
+    """A table storing gene coordinates and optional cluster probabilities.
     """
 
     sequence_id: List[str] = field(default_factory = list)
